@@ -1,5 +1,5 @@
 from subs2cia.sources import Stream
-from subs2cia.ffmpeg_tools import ffmpeg_demux
+from subs2cia.ffmpeg_tools import ffmpeg_demux, ffmpeg_trim_audio_clip_atrim_encode, ffmpeg_get_frame_fast
 
 import logging
 import pysubs2 as ps2  # for reading in subtitles
@@ -9,6 +9,73 @@ import ffmpeg
 import re
 import copy
 
+from typing import List, Union
+
+
+def overlap_range(range1: List[int], range2: List[int]):
+    assert len(range1) == len(range2) == 2
+    if range1[0] < range2[0] < range1[1] or range1[0] < range2[1] < range1[1] or range2[0] < range1[0] < range2[1] or \
+        range2[0] < range1[1] < range2[1]:
+        return True
+    return False
+
+
+def ssaevent_trim(event: ps2.SSAEvent, ir: List[int]):
+    r"""
+    Given an event that overlaps with ir
+    if subtitle is only on one side of IR
+        trim subtitle to not overlap IR
+    if subtitle is on both sides of IR
+        split subtitle into two and trim each
+    if subtitle is entirely inside IR
+        drop it
+    :param event: SSAEvent to trim/split+trim/drop
+    :param ir: Ignore range, two integers representing milliseconds
+    :return: List of trimmed events. There may be zero, one, or two events returned
+    """
+
+    assert len(ir) == 2
+    trimmed = []
+    if ir[0] <= event.start < ir[1] and ir[0] < event.end <= ir[1]:
+        # event falls completely inside an ignore range
+        return trimmed
+    if ir[0] <= event.start < ir[1] and ir[1] < event.end:
+        # event starts inside ignore range, ends outside of it
+        event.start = ir[1]
+        trimmed.append(event)
+        return trimmed
+    if event.start < ir[0] and ir[0] < event.end <= ir[1]:
+        # event starts outside of ignore range, ends inside of it
+        event.end = ir[0]
+        trimmed.append(event)
+        return trimmed
+    if event.start < ir[0] and ir[1] < event.end:
+        # split into two pieces
+        event2 = copy.deepcopy(event)
+        event.end = ir[0]
+        trimmed.append(event)
+        event2.start = ir[1]
+        trimmed.append(event2)
+        return trimmed
+    # should never get here
+    assert False
+
+
+def ignore_nibble(ignore_ranges: List[List[int]], e: ps2.SSAEvent):
+    r"""
+    Given a set of ignore_ranges and a SSAEvent, determine if the event falls in an IR and if it does, trim it
+    :param ignore_ranges: List of IRs
+    :param e: SSAEvent of interest
+    :return: List of SSAEvents
+    """
+    for ir in ignore_ranges:
+        assert len(ir) == 2
+        if overlap_range(ir, [e.start, e.end]):
+            trimmed = ssaevent_trim(e, ir)
+            # assert(trimmed is not None)
+            return trimmed
+    return [e]
+
 
 class SubGroup:
     def __init__(self, events: [ps2.SSAEvent], ephemeral: bool, threshold: int, padding: int):
@@ -37,6 +104,10 @@ class SubGroup:
 
     @property
     def group_range(self):
+        r"""
+        Subtitle group start, end with padding
+        :return: [range_start, range_end]
+        """
         if self.contains_only_ephemeral:
             return [self.events_start, self.events_end]
         grange = self.padding
@@ -44,6 +115,10 @@ class SubGroup:
 
     @property
     def group_limits(self):
+        r"""
+        Subtitle group start/end extended with padding and threshold
+        :return: [limit_start, limit_end]
+        """
         if self.contains_only_ephemeral:
             return [self.events_start, self.events_end]
         limit = self.threshold/2 + self.padding  # divide by two: threshold is distance to next group
@@ -54,9 +129,16 @@ class SubGroup:
         return s
 
 
-# TODO
 class SubtitleManipulator:
-    def __init__(self, subpath: Path, threshold: int, padding: int):
+    def __init__(self, subpath: Path, threshold: int, padding: int, ignore_range: Union[List[List[int]], None], audio_length: int):
+        r"""
+        Class for subtitle manipulation
+        :param subpath: Path to pysubs2-compatible subtitle file
+        :param threshold: in milliseconds
+        :param padding: in milliseconds
+        :param ignore_range: list of ranges. each range is a list of two tuples. each tuple contains two values, a "sign" and a "duration".
+        :param audio_length: How long the audio is in milliseconds.
+        """
         self.subpath = subpath
         self.ssadata = None
         self.condensed_ssadata = None
@@ -67,6 +149,34 @@ class SubtitleManipulator:
         self.threshold = threshold
         self.padding = padding
 
+        self.audio_length = audio_length
+
+        self.ignore_range = None
+        if ignore_range is not None:
+            self.ignore_range = []
+            for irange in ignore_range:
+                assert(len(irange) == 2)
+                to_append = []
+                for idx, (sign, duration) in enumerate(irange):
+                    if sign == "+":
+                        if idx == 0:
+                            raise AssertionError("Can't have duration '+' as first range value, "
+                                                 "must be second range value "
+                                                 "(e.g. -I 1m +1m30s)")
+                        to_append.append(to_append[0] + duration)
+                    elif sign == "e":
+                        to_append.append(int(self.audio_length - duration))
+                    elif sign == "":
+                        to_append.append(duration)
+                    else:
+                        # shouldn't get here, re.findall should strip anything unexpected out
+                        raise AssertionError(f"SubtitleManipulator received unexpected sign ({sign})")
+                if not to_append[0] < to_append[1]:
+                    raise AssertionError(f"Ignore range '{irange[0][0]}{irange[0][1]}ms {irange[1][0]}{irange[1][1]}ms'"
+                                         f" is invalid: end of range "
+                                         f"({to_append[1]}ms) is before start of range ({to_append[0]}ms)")
+                self.ignore_range.append(to_append)
+
     def load(self, include_all, regex):
         if not self.subpath.exists():
             logging.warning(f"Subtitle file {self.subpath} does not exist")
@@ -86,8 +196,15 @@ class SubtitleManipulator:
         self.ssa_events = self.ssadata.events
         self.ssa_events.sort(key=lambda x: x.start)
 
+        pool = self.ssa_events
         self.groups = []
-        for e in self.ssa_events:
+        while len(pool) > 0:
+            e = pool.pop(0)
+            if self.ignore_range is not None:
+                if any([overlap_range(ir, [e.start, e.end]) for ir in self.ignore_range]):
+                    trimmed = ignore_nibble(self.ignore_range, e)
+                    pool = trimmed + pool
+                    continue  # trimmed events may still overlap other ranges, will need to retest
             self.groups.append(SubGroup([e], ephemeral=not is_dialogue(e, include_all, regex),
                                         threshold=self.threshold,
                                         padding=self.padding))
